//	-------------------------------------------------------------------------
//
//	ACCESSRY.HPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains defines for the Accessory base class,
//					and some simple derived classes - Button, Bar & Box.
//					Also functions for installing accessory bitmaps, etc.
//					eg. InstallGUI() and ExitGUI()
//
//	-------------------------------------------------------------------------

#ifndef __accessry_hpp
#define __accessry_hpp

#include "graphics.hpp"
#include "event.hpp"

//	-------------------------------------------------------------------------
//
//	CLASS: 		Accessory
//	DESC: 		base class for all objects contained inside an event handler
//      		Examples for a simple gui include Buttons, Bars, Boxes, etc.
//
//	NOTES:		ALL ACCESSORIES _SHOULD_ RESPOND TO BOTH KEYBOARD & MOUSE!!!!
//
//	-------------------------------------------------------------------------

// options for all Accessory's
#define ac_NoOption				0x0000		// no option (0 suffices)
#define ac_SaveUnderAccessory	0x0001		// defines save image.
#define ac_IsNotSelectable		0x0002		// cannot be selected

class Accessory
	{
	protected:
		uint 	returnEvent;  		// command event returned from HandleEvent()

		uchar 	fgcolor;			// foreground color
		uchar   bgcolor;			// background color

		grImage* 	savedImage; 		// ptr to saved image underneath our item

		uint 	selected_options; 	// the options set for this item

		uint 	active_bs;			// active border style
		uint 	inactive_bs;		// inactive border style

	public:
		bool	isActive;			// the accessory is currently active
		bool 	isSelectable;		// can be selected by mouse/keyboard
		bool 	isSelected;			// the accessory is currently selected

		grRect 	r;					// rectangle dimensions used by the
									// accessory.
		grRect	inset_r;			// the rectangle dimensions inside the
									// border, etc.


		Accessory(int x1,int y1,int x2,int y2,uchar fg,uchar bg,
				  uint active_border,uint inactive_border,
				  uint options,uint retevent);

		virtual ~Accessory();

		// a quick check to see whether or not mouse is inside 'r'
		bool	MouseInside(grPoint p) { if ((p.x>=r.left)&&(p.x<=r.right)&&
											(p.y>=r.top)&&(p.y<=r.bottom))
											return true;
										 else
											return false; }

		// adjusts the coordinates of the accessory (assumes not onscreen)
		virtual void AdjustCoords(int dx,int dy);

		// lets accessory know that it has been selected
		virtual void Select();

		// deselect's the accessory
		virtual void DeSelect();

		// animate the accessory
		virtual void Animate();

		// draw the accessory (first time in)
		virtual void Draw();

		// update the accessory on the screen
		virtual void Refresh();

		// handle any events required by the accessory!
		virtual void HandleEvent(Event &e);
	};

//	-------------------------------------------------------------------------
//
//	FUNC:		InstallGUI, ExitGUI
//
//	DESC:		Installs or exits the basic GUI, including a number of
//				pre-defined buttons, vertical & horizontal scroll bars,
//				a text input bar, a vertical scrolling text box, and so on.
//
//	NOTES:		ExitGUI _must_ be called if you use InstallGUI
//
//	-------------------------------------------------------------------------

void InstallGUI();
void ExitGUI();

// images declared globally
extern grImage*  	GUI_BITMAP[];	// the bitmap storage area

#endif