//	-------------------------------------------------------------------------
//
//	DIR.CPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file implements the DirString object as well as
//					some directory handling functions.
//
//	-------------------------------------------------------------------------

#include "dir.hpp"
//#include "graphics.hpp"

#include "fcntl.h"
#include "dos.h"
#include "direct.h"
#include "sys\stat.h"
#include "errno.h"
//#include "dir.h"

void DirString::SetFileName(const char* st)
	{
	if (len>0)
		{
		int start = ScanLeft(len,'\\')+1;
		Delete(start,len-start);
		}
	Append(st);
	}

void DirString::SetDrive(const char* st)
	{
	if (len>0)
		{
		int end = ScanRight(0,':') + 1;
		Delete(0,end);
		Insert(0,st);
		}
	else
		Append(st);
	}

void DirString::SetPath(const char* st)
	{
	int start = ScanRight(0,':') + 1;
	int end = ScanLeft(len,'\\');

	if (start<end) Delete(start+1,end-start);

	Insert(start,st);

	// if there is no '\' at the start of the path, put it in now!
	if (Contents()[2]!='\\') Insert(2,'\\');

	// if there is no '\' at the start of the path, put it in now!
	// (except for when we are in the root directory!!!)
//	DBG->Display("before insert == >%s<",Contents());
//	if ((st[0]!='\\')&&(strlen(st)!=0)) Insert(start,'\\');
//	DBG->Display("output string == >%s<",Contents());
	}

void DirString::SetDirectory(const char* st)
	{
	int start = ScanLeft(Length(),'\\') + 1;

	// if there is no '\' at the end of the st, put it in
	if (st[strlen(st)]!='\\') Insert(start,'\\');

	Insert(start,st);
	}

String* DirString::GetFileName()
	{
	int end = Length();
	int start = ScanLeft(end,'\\')+1;
	String* tmpstr = SubStr(start,end);
	return tmpstr;
	}

String* DirString::GetDrive()
	{
	int end = ScanRight(0,':');
	String* tmp = SubStr(0,end);
	return tmp;
	}

String* DirString::GetPath()
	{
	int start = ScanRight(0,':') + 1;
	int end = ScanLeft(Length(),'\\');
	String* tmp = SubStr(start,end);
	return tmp;
	}

String* DirString::GetDirectory()
	{
	int end = ScanLeft(Length(),'\\') - 1;
	int start = ScanLeft(end,'\\') + 1;
	String* tmp = SubStr(start,end);
	return tmp;
	}

// other functions for use with files & directories

bool dir_FileExists(DirString* pathname)
	{
	int handle = open(pathname->Contents(), O_RDWR, S_IREAD | S_IWRITE );
	if (handle == -1) return false;
	else { close(handle); return true; }
	}

int dir_FilesInDir(uint attrib)
	{
/*
FA_RDONLY  Read-only attribute
FA_HIDDEN  Hidden file
FA_SYSTEM  System file
FA_LABEL   Volume label
FA_DIREC   Directory
FA_ARCH    Archive

_A_RDONLY  Read-only attribute
_A_HIDDEN  Hidden file
_A_SYSTEM  System file
_A_VOLID   Volume label
_A_SUBDIR  Directory
_A_ARCH    Archive
findfirst
*/
	struct find_t ffblk;
	int attr = 0;
	switch (attrib)
		{
		case	dir_DirectoriesOnly:	attr |= _A_SUBDIR; break;
		case 	dir_FilesOnly:			break;
		case	dir_AllFiles:			attr |= _A_RDONLY | _A_HIDDEN | _A_SYSTEM | _A_VOLID | _A_SUBDIR | _A_ARCH;
		}

	int done;
	int cnt=0;
	done = _dos_findfirst("*.*",attr,&ffblk);
	cnt++;

	while (!done)
		{
		done = _dos_findnext(&ffblk);
		cnt++;
		}
	return cnt;
	}

String* dir_ReadDirectory(uint attrib)
	{
	find_t fb;
	int attr = 0;
	int done;
	String* dirstr = new String(13*1024);

	switch (attrib)
		{
		case	dir_DirectoriesOnly:	attr |= _A_SUBDIR; break;
		case 	dir_FilesOnly:			attr |= 32; break;
		case	dir_AllFiles:			attr |= _A_RDONLY | _A_HIDDEN | _A_SYSTEM | _A_VOLID | _A_SUBDIR | _A_ARCH;
		}

	dirstr->Set("");
	done = _dos_findfirst("*.*",attr,&fb);

//	int cnt = 0;
	while (!done)
		{
		if (fb.attrib==attr)   // NOTE:: BUGFIX... check attrib manually & findnext doesn't??
			{
//			DBG->Display("filename #%d == >%s<",cnt++,fb.ff_name);
			dirstr->Append(fb.name);
			dirstr->Append("/");
/*if (attrib==dir_DirectoriesOnly) {
grSetColor(BLACK);
grDrawFilledRect(500,20+cnt*10,600,20+cnt*10+10);
grSetColor(LIGHTRED);
grDrawString(500,20+cnt*10,">%s<",fb.ff_name);
}
if (attrib==dir_FilesOnly) {
grSetColor(BLACK);
grDrawFilledRect(0,20+cnt*10,100,20+cnt*10+10);
grSetColor(LIGHTGREEN);
grDrawString(0,20+cnt*10,">%s<",fb.ff_name);

			cnt++;
}
*/			}
		done = _dos_findnext(&fb);
		}
/*if (attrib==dir_DirectoriesOnly) {
grSetColor(BLACK);
grDrawFilledRect(500,20+cnt*10,600,20+cnt*10+20);
grSetColor(LIGHTRED);
grDrawString(500,20+cnt*10,"DIRS == %d",cnt);
}
if (attrib==dir_FilesOnly) {
grSetColor(BLACK);
grDrawFilledRect(0,20+cnt*10,100,20+cnt*10+20);
grSetColor(LIGHTGREEN);
grDrawString(0,20+cnt*10,"FILES == %d",cnt);
}
*/
	if (dirstr->Length()>0)
		dirstr->Delete(dirstr->Length()-1,1);
//	if (heapcheck()==_HEAPCORRUPT) DBG->Display("HEAP IS CORRUPT! (ReadDirectory)");

	return dirstr;
	}

String* dir_GetDriveNames()
	{
	uint current_disk;
	_dos_getdrive(&current_disk);

	uint num_drives;
	_dos_setdrive(current_disk, &num_drives);

	String* diskstr = new String(3 * 26);

	diskstr->Set("");
	for (int i=0;i<num_drives;i++)
		{
		diskstr->Append((char)65+i);
		diskstr->Append(":/");
		}
	diskstr->Delete(diskstr->Length(),1);

	return diskstr;
	}

String* dir_GetCurrentPath()
	{
	String* pathstr = new String(512);
	// add current path  "\CURRENT\DIRECTRY\"
	getcwd(pathstr->Contents(),pathstr->MaxLength());
	// and return our string
	return pathstr;
	}