//	--------------------------------------------------------------------------
//
//	FONTS.CPP
//
// *NOTE:: the font is stored in the same order as the ascii set
// but the first character is the space key 32, the list is contiguous
// after this for 95 characters.
//
// 		the font character table.
//		const uchar	character_table[] = // 39 = "'", 92 = "\"
//		{
//		' ','!','"','#','$','%','&',39,'(',')','*','+',',','-','.','/',
//		'0','1','2','3','4','5','6','7','8','9',':',';','<','=','>','?',
//
//		'@','A','B','C','D','E','F','G','H','I','J','K','L','M',
//		'N','O','P','Q','R','S','T','U','V','W','X','Y','Z','[',92,']','^','_',
//
//		'`','a','b','c','d','e','f','g','h','i','j','k','l','m',
//		'n','o','p','q','r','s','t','u','v','w','x','y','z','{','|','}','~'
//		};
//
//	--------------------------------------------------------------------------

#include	"fonts.hpp"

#include	"stdarg.h"
#include	"stdio.h"

FontResourceFile::FontResourceFile(DirString* pathname,uint options,long num_resources)
					: ResourceFile(pathname,options,num_resources,resource_BITMAP_RESOURCE)
	{
	}

FontResourceFile::FontResourceFile(	const char* pathname,uint options,long num_resources)
					: ResourceFile(	pathname,options,num_resources,resource_BITMAP_RESOURCE)
	{
	}

//	-------------------------------------------------------------------------
//
//	FUNC:	SizeOfResource
//	DESC:	use to malloc the size of image in a resource
//
//	-------------------------------------------------------------------------
ulong FontResourceFile::SizeOfResource (long resource_num)
	{
	// read the resource index
	ReadResourceIndex( resource_num );

	// if the size is zero, there is no said resource
	if (index.size==0) return 0;

	// size is the size of the resource itself
	ulong size = index.size;

	return size;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ReadResource
//	DESC:       reads a resource of 'size' bytes, and places them
//				in an image 'buffer'
//
//	-------------------------------------------------------------------------
int FontResourceFile::ReadResource(long resource_num,char* buffer)
	{
	// skip out if we're write-only
	if (fileOptions&dosfile_WRITEONLY) return fileError = resource_FileIsWriteOnly;

	// read the resource status
	if ((fileError = ReadResourceIndex( resource_num )) != resource_NoError) return fileError;

	// if the size is zero, there is no said resource
	if (index.size==0) return fileError = resource_ResourceNotFound;

	// seek to the start of the resource
	if ((fileError = Seek( index.location )) != resource_NoError) return fileError;

	// read the resource
	fileError = DOSFile::Read(buffer, index.size );

	// return our last file error
	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		AddResource
//	DESC:		Will add a character of a font to a font resource file
//				If you want to modify your resource file you have to delete
//				it and rewrite all the resources in, in the correct order
//
//	-------------------------------------------------------------------------
int FontResourceFile::AddResource(char* srce, ulong resourcesize)
	{
	// write the resource into this resourcefile
	int fileError = WriteResource(NumResources(),resourcesize,srce);

//printf("1 written resource ptr:%p size:%lu",srce,resourcesize);printf("\n");getch();

	return fileError;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		LoadAllResources
//	DESC:		malloc each data item then load each data item of a resource
//
//	-------------------------------------------------------------------------
char** FontResourceFile::LoadAllResources()
	{
	// get number of resources to load
	int num_resources = NumResources();

	// malloc memory for list of pointers to each resource
	grImage** datalist = (grImage**) malloc(sizeof(grImage*) * num_resources);

	int i;
	for (i=0;i<num_resources;i++)
		{
		// malloc memory for each resource
		datalist[i] = (grImage*) malloc(SizeOfResource(i)+1);

		// read resource into memory where datalist[i] is pointing
		fileError = ReadResource (i, (char*) datalist[i]);
		// on error
		if (fileError)
			grDrawString(0,400,"could not read resource error #%d",fileError);
		}

	// return pointer to whole list
	return (char**)datalist;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ClearAllResources
//	DESC:		free each data item of a resource
//
//	-------------------------------------------------------------------------
void FontResourceFile::FreeAllResources(char** datalist)
	{
	int i;
	for (i=NumResources()-1;i>=0;i--)
		// free each resource
		free(datalist[i]);

	// free whole list of pointers
	free (datalist);
	}
