//	-------------------------------------------------------------------------
//
//	KEYBOARD.HPP
//
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains defines for a keyboard handler which
//					is an ISR generating events into the EVENTQUEUE
//
//	-------------------------------------------------------------------------

#ifndef __keyboard_hpp
#define __keyboard_hpp

#include "keydefns.hpp"
#include "stddefns.hpp"

// keyboard state modifiers

#define keyboard_LEFT_SHIFT		0x0001
#define keyboard_RIGHT_SHIFT	0x0002
#define keyboard_LEFT_CTRL		0x0004
#define keyboard_RIGHT_CTRL		0x0008
#define keyboard_LEFT_ALT		0x0010
#define keyboard_RIGHT_ALT		0x0020

#define keyboard_SHIFT			(keyboard_LEFT_SHIFT | keyboard_RIGHT_SHIFT)
#define keyboard_CTRL			(keyboard_LEFT_CTRL | keyboard_RIGHT_CTRL)
#define keyboard_ALT			(keyboard_LEFT_ALT | keyboard_RIGHT_ALT)

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_IsReleased()
//	DESC:		returns true if the specified keypress is a key_released event
//
//	-------------------------------------------------------------------------
bool keyboard_IsReleased( uint keyvalue );

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_IsPressed()
//	DESC:		returns true if the specified keypress is a key_pressed event
//
//	-------------------------------------------------------------------------
bool keyboard_IsPressed( uint keyvalue );

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_ASCII()
//	DESC:		returns the ascii value of a keyboard return value. Also
//				takes into account the status of the SHIFT keys.
//				eg. keyboard_ASCII( RET_DN, SHIFT ) == 13
//
//	-------------------------------------------------------------------------
uchar keyboard_ASCII( uint keyvalue, uint modifiers );

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_GetState()
//	DESC:		returns the current state of the shift/alt/ctrl keys
//
//	-------------------------------------------------------------------------
uint keyboard_GetState();

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_SetAutoRepeat()
//	DESC:		sets the delay for the auto repeat on key_dn
//
//	-------------------------------------------------------------------------
void keyboard_SetAutoRepeat( uint rate );

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_GetEvent()
//	DESC:		checks to see if any keys have been pressed & generates
//				an appropriate event into the EVENT QUEUE
//
//	-------------------------------------------------------------------------
void keyboard_GetEvent();

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_InstallHandler()
//	DESC:		installs our custom keyboard handler.
//
//	-------------------------------------------------------------------------
void keyboard_InstallHandler();

//	-------------------------------------------------------------------------
//
//	FUNC:		keyboard_RemoveHandler()
//	DESC:		Removes our custom keyboard handler & sets the handler back
//				to the original (BIOS/DOS/???) keyboard handler.
//
//	-------------------------------------------------------------------------
void keyboard_RemoveHandler();

#endif