//	-------------------------------------------------------------------------
//
//	MOUSE.CPP
//
//	-------------------------------------------------------------------------
//
//	AUTHOR:			Darren J. Dwyer
//
//	DESCRIPTION:	This file contains code for a mouse handler which
//					mimicks an ISR generatating events into the EVENTQUEUE
//
//	-------------------------------------------------------------------------

//#include "misc.hpp"
#include "mouse.hpp"
//#include "keyboard.hpp"
#include "event.hpp"
//#include "eventmgr.hpp"

#include "graphics.hpp"

#include "mem.h"
#include "i86.h"
#include "conio.h"
#include "stdio.h"
#include "stdlib.h"

// global variables used by the following routines

uint 	mouse_xdist;    	// x distance travelled before moveevents generated
uint 	mouse_ydist;		// y distance travelled before moveevents generated
bool 	mouse_MoveEventsOn = false;	// whether or not to generate move events
bool 	mouseInstalled = false;		// whether or not the mouse functions are installed
bool	mouseIsHidden  = true;		// whether or not the mouse is hidden
mouse_Status	mouse;				// current mouse status
mouse_Status 	lastmouse;			// last mouse status
uint    lastmousex;					// last cursor x position
uint	lastmousey;					// last cursor y position

MouseCursor		ARROW_CURSOR = {	24,24,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

								0,0,8,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 7,7,7,7, 7,7,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 7,7,7,7, 7,7,7,7, 15,15,0,0, 0,0,0,0, 0,0,0,0,

								0,0,8,7, 7,7,7,15, 15,15,15,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,

								0,0,8,7, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 8,7,7,7, 7,7,7,15, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,7, 8,8,7,7, 7,7,7,7, 15,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,8,8, 0,0,8,7, 7,7,7,7, 7,15,0,0, 0,0,0,0, 0,0,0,0,

								0,0,8,0, 0,0,0,8, 7,7,7,7, 7,7,15,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 8,7,7,7, 7,7,8,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 0,8,7,7, 7,8,0,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 0,0,8,7, 8,0,0,0, 0,0,0,0, 0,0,0,0,

								0,0,0,0, 0,0,0,0, 0,0,0,8, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0	};

MouseCursor		DummyCursor = { 0x0010, 0x0010,	// 16x16
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,
								0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
								0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,

								0,0,0,0, 0,0,4,4, 4,4,0,0, 0,0,0,0,
								0,9,0,0, 0,0,4,4, 4,4,0,0, 0,9,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,9,0,0, 0,0,0,0, 0,0,0,0, 0,9,0,0,

								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,9,0,9, 0,9,0,9, 0,9,0,9, 0,9,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
								0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0 };

MouseCursor*	mouse_Cursor = &DummyCursor;
grImage*		mouse_UnderCursor = NULL;


MouseEvent mouse_EventInfo;			// mouse event information

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_EvableMoveEvents()
//	DESC:		upon calling this function, mouse_GetEvent() will generate
//				mb_move events. (Note: this may cause a large number of
//				events to be placed in the event queue!!!)
//
//	-------------------------------------------------------------------------
void mouse_EnableMoveEvents()
	{
	mouse_MoveEventsOn = true;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DisableMoveEvents()
//	DESC:		stops mouse_GetEvent from generating mouse move events into
//				the EVENT QUEUE.
//
//	-------------------------------------------------------------------------
void mouse_DisableMoveEvents()
	{
	mouse_MoveEventsOn = false;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetMoveThreshold()
//	DESC:		Use this to set the desired movement (in pixels) before
//				mouse move events are generated by mouse_GetEvent()
//
//	PARAMS:		xthresh - the distance required in x direction
//				ythresh - the distance required in y direction
//
//	-------------------------------------------------------------------------
void mouse_SetMoveThreshold(uint xthresh,uint ythresh)
	{
	mouse_xdist = xthresh;
	mouse_ydist = ythresh;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetPos()
//	DESC:		returns the current position of the mouse as a point
//
//	-------------------------------------------------------------------------
grPoint mouse_GetPos()
	{
	grPoint p;

	p.x = mouse.x;
	p.y = mouse.y;

	return p;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetEvent()
//	DESC:		checks to see if the mouse has changed status at all
//				and generates an appropriate event into the EVENTQUEUE
//
//	-------------------------------------------------------------------------

void mouse_GetEvent()
	{
	if (!mouseInstalled) return;

	// predefine most of our event (easier later on)
	mouse_EventInfo.event = 0;
//	mouse_EventInfo.modifiers = keyboard_GetState();
	mouse_EventInfo.where.x = mouse.x;
	mouse_EventInfo.where.y = mouse.y;

	// if the left mouse button is currently down
	if (mouse.leftBut == mouse_PRESSED)
		// generate an event that mouse is down (AUTO REPEAT CAPABILITY)
		{
		mouse_EventInfo.event = mb_leftdn;
//		if (!EVENTMGR->EventInQueue(ev_mouse,&mouse_EventInfo))
//			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
		}
	else
		// if the mouse was down previously
		if (lastmouse.leftBut == mouse_PRESSED)
			// generate an event that mouse is now up
			{
			mouse_EventInfo.event = mb_leftup;
//			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
			}

	// do same for the right button
	if (mouse.rightBut == mouse_PRESSED)
		{
		mouse_EventInfo.event = mb_rightdn;
//		if (!EVENTMGR->EventInQueue(ev_mouse,&mouse_EventInfo))
//			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
		}
	else
		if (lastmouse.rightBut == mouse_PRESSED)
			{
			mouse_EventInfo.event = mb_rightup;
//			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
			}

	// and for the centre button
	if (mouse.middleBut == mouse_PRESSED)
		{
		mouse_EventInfo.event = mb_middledn;
//		if (!EVENTMGR->EventInQueue(ev_mouse,&mouse_EventInfo))
//			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
		}
	else
		if (lastmouse.middleBut == mouse_PRESSED)
			{
			mouse_EventInfo.event = mb_middleup;
//			EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
			}

	// if we are tracking movements
	if (mouse_MoveEventsOn)
		{
		// if the mouse has moved a set distance, generate a move event
		if ((diff(lastmouse.x,mouse.x)>mouse_xdist)&&
			(diff(lastmouse.y,mouse.y)>mouse_ydist))
				{
				mouse_EventInfo.event = mb_move;
//				EVENTMGR->GenEvent(0,ev_mouse,&mouse_EventInfo);
				}
		}

	// save our info for the next time in.....
	lastmouse.leftBut  = mouse.leftBut;
	lastmouse.rightBut = mouse.rightBut;
	lastmouse.middleBut= mouse.middleBut;
	lastmouse.x = mouse.x;
	lastmouse.y = mouse.y;
	}

void mouse_Hide()
	{
	if (!mouseInstalled) return;

	if (!mouseIsHidden)
		{
		uint x1 = lastmousex;
		uint y1 = lastmousey;
		uint x2 = x1 + mouse_UnderCursor->hdr.width -1;
		uint y2 = y1 + mouse_UnderCursor->hdr.height -1;

		grPutImage(x1,y1,x2,y2,(grImage*) mouse_UnderCursor);

		mouseIsHidden = true;
		}
	}

void mouse_Show()
	{
	if (!mouseInstalled) return;

	if (mouseIsHidden)
		{
		uint x1 = mouse.x;
		uint y1 = mouse.y;
		uint x2 = x1 + mouse_Cursor->hdr.width - 1;
		uint y2 = y1 + mouse_Cursor->hdr.height - 1;

		//printf("\nmouse coords are %.3d,%.3d : %.3d,%.3d",x1,y1,x2,y2);

		grGetImage(x1,y1,x2,y2,mouse_UnderCursor);
		grOverlayImage(x1,y1,x2,y2,(grImage*) mouse_Cursor);

		lastmousex = x1;
		lastmousey = y1;

		mouseIsHidden = false;
		}
	}

void mouse_UpdateCursor()
	{
	if ((mouse.x == lastmousex)&&
		(mouse.y == lastmousey)) return;

	mouse_Hide();
	mouse_Show();
	}

void mouse_SetCursor(int hotx,int hoty,MouseCursor* curs)
	{
	mouse_Cursor = curs;
	}

#pragma off (check_stack)
void _loadds far mouse_ISR(int max, int mbx, int mcx, int mdx, int msi, int mdi)
		{
#pragma aux mouse_ISR parm [EAX] [EBX] [ECX] [EDX] [ESI] [EDI]
		mouse.leftBut  = (((ushort) mbx & 0x0001)==0x0001);
		mouse.rightBut = (((ushort) mbx & 0x0002)==0x0002);
		mouse.middleBut= (((ushort) mbx & 0x0003)!=0x0000);	// left + right

		mouse.x  	   = (ushort) mcx / 8;
		mouse.y  	   = (ushort) mdx / 8;
//		mouse.mickeysx = (signed short) msi;
//		mouse.mickeysy = (signed short) mdi;

		// note: if bit 0 of EAX set, mouse is moving.... so update cursor
//		if (max&1) mouse_UpdateCursor();
		}
void mouse_ISR_end() {} // Dummy function so we can calculate sizeof(mouse_ISR)
#pragma on (check_stack)


bool mouse_DriverLoaded()
	{
	REGS regs;
	memset(&regs,0,sizeof(regs));

	// try resetting mouse driver
	regs.w.ax = 0;
	int386(MOUSE_INT, &regs, &regs);

	// and return true if driver is loaded
	return (regs.w.ax==0xFFFF);
	}


void mouse_InstallHandler()
	{
	if (mouseInstalled) return;

	// if there is a driver in memory
	if (mouse_DriverLoaded())
		{
		//printf("activating mouse");
		// temp data
		REGS regs;
		SREGS sregs;
		memset(&regs,0,sizeof(regs));
		memset(&sregs,0,sizeof(sregs));

		// call mouse MOUSE BIOS function 0x0C
		regs.w.ax 	= 0x000C;
					// move    leftpr   leftrel  rightpr  rightrel  midpr	midrel
		regs.w.cx 	= 0x0001 + 0x0002 + 0x0004 + 0x0008 + 0x0010;  // + 0x0020 + 0x0040
		regs.x.edx 	= FP_OFF(mouse_ISR);
		sregs.es    = FP_SEG(mouse_ISR);
		int386x(MOUSE_INT, &regs, &regs, &sregs);

		// set mouse mickey/pixel ratios
		regs.w.ax	= 0x000F;
		regs.w.cx	= 8 / 8; 	// x ratio == 8 (default) / 8 (due to inability to set 640 * 480 response)
		regs.w.dx	= 16 / 8;	// y ratio == 16 (default) / 8 (due to inability to set 640 * 480 response)
		int386(MOUSE_INT, &regs, &regs);

		// set mouse horizontal range to 0-639
		regs.w.ax	= 0x0007;
		regs.w.cx	= 0x0000;
		regs.w.dx	= 640 * 8 - 1; 	// (due to inability to set 640 * 480 response)
		int386(MOUSE_INT, &regs, &regs);

		// set mouse vertical range to 0-479
		regs.w.ax	= 0x0008;
		regs.w.cx	= 0x0000;
		regs.w.dx	= 480 * 8 - 1;	// (due to inability to set 640 * 480 response)
		int386(MOUSE_INT, &regs, &regs);

		// set mouse to location 319,239 (centre of screen)
		regs.w.ax	= 0x0004;
		regs.w.cx	= 320 * 8 - 1;	// * 8 again
		regs.w.dx	= 240 * 8 - 1;	// * 8 again
		int386(MOUSE_INT, &regs, &regs);

		// create room for whats under cursor.... (max. size == 32 * 32)
		mouse_UnderCursor = (grImage*) malloc(grImageSize(0,0,31,31));

		// setup our initial mouse data
		memset(mouse_UnderCursor,0,grImageSize(0,0,31,31));
		memset(&mouse,0,sizeof(mouse));
		mouse.x = 319;
		lastmousex = 319;
		mouse.y = 239;
		lastmousey = 239;

		// mouse handler is now installed
		mouseInstalled = true;
		}
//	else
		//printf("\ncannot activate mouse");
	}

void mouse_RemoveHandler()
	{
	if (!mouseInstalled) return;

	// get rid of what was under cursor
	free(mouse_UnderCursor);

	REGS regs;
	memset(&regs,0,sizeof(regs));

	// we can remove the ISR by calling the mouse init function (0x00)
	regs.w.ax = 0x0000;
	int386(MOUSE_INT, &regs, &regs);
	}