//	-------------------------------------------------------------------------
//
//	MOUSE.HPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file contains defines for a mouse handler which
//					mimicks an ISR generatating events into the EVENTQUEUE
//
//	-------------------------------------------------------------------------

#ifndef __mouse_hpp
#define __mouse_hpp

#include "graphics.hpp"
#include "stddefns.hpp"

#define mouse_PRESSED		1
#define mouse_RELEASED		0

// the mouse information structure
struct mouse_Status
	{
	WORD		leftBut;
	WORD		rightBut;
	WORD		middleBut;

	WORD		x;
	WORD		y;
	};

// note: a mouse cursor is based on a grImage, but it _must_ always be a
//		 32 * 32 pixel (or less) image!!

#define MouseCursor		grImage

#define MOUSE_INT		0x33

// defines used to interpret the mouse
//
// eg. 		if (e.what==ev_mouse)
//          	switch (e.mouse.event) {
//					case mb_leftdn:		printf("left mouse was pressed\n"); break;
//					case mb_leftup:		printf("left mouse was released\n"); break;
//					case mb_move:		printf("mouse is moving!!!\n"); break; }

#define		mb_leftdn		1
#define		mb_leftup		2
#define		mb_rightdn		3
#define		mb_rightup		4
#define 	mb_middledn		5
#define		mb_middleup		6
#define		mb_move			7

extern	MouseCursor		ARROW_CURSOR;
//extern 	MouseCursor		HOURGLASS_CURSOR

void mouse_Show();
void mouse_Hide();
void mouse_UpdateCursor();
void mouse_SetCursor(int hotx,int hoty,MouseCursor* curs);

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_EvableMoveEvents()
//	DESC:		upon calling this function, mouse_GetEvent() will generate
//				mb_move events. (Note: this may cause a large number of
//				events to be placed in the event queue!!!)
//
//	-------------------------------------------------------------------------
void mouse_EnableMoveEvents();

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_DisableMoveEvents()
//	DESC:		stops mouse_GetEvent from generating mouse move events into
//				the EVENT QUEUE.
//
//	-------------------------------------------------------------------------
void mouse_DisableMoveEvents();

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_SetMoveThreshold(uint thresh)
//	DESC:		Use this to set the desired movement (in pixels) before
//				mouse move events are generated by mouse_GetEvent()
//
//	PARAMS:		xthresh - the distance required in x direction
//				ythresh - the distance required in y direction
//
//	-------------------------------------------------------------------------
void mouse_SetMoveThreshold(uint xthresh,uint ythresh);

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetPos()
//	DESC:		returns the current position of the mouse as a point
//
//	-------------------------------------------------------------------------
grPoint mouse_GetPos();

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_GetEvent()
//	DESC:		checks to see if the mouse has changed status at all
//				and generates an appropriate event into the EVENT QUEUE
//
//	-------------------------------------------------------------------------
void mouse_GetEvent();

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_InstallHandler()
//	DESC:		checks to see if a mouse driver is active and if so will
//				install an interrupt handler to respond to mouse input
//
//	-------------------------------------------------------------------------
void mouse_InstallHandler();

//	-------------------------------------------------------------------------
//
//	FUNC:		mouse_RemoveHandler()
//	DESC:		removes previously installed mouse handler
//
//	-------------------------------------------------------------------------
void mouse_RemoveHandler();

#endif