//	-------------------------------------------------------------------------
//
//	PCX.HPP
//
//	-------------------------------------------------------------------------
#ifndef		__pcx_hpp
#define		__pcx_hpp

#include		"dir.hpp"
#include		"graphics.hpp"

//	PCX_Header:	a header for a bitmap using standard PCX graphics
//					compression format
struct PCX_imageHeader
	{
	BYTE		manufacturer;		// 10
	BYTE		version;			// 5
	BYTE		encoding;			// 1 for Run Length Encoding (RLE)
	BYTE		bits_per_pixel;		// 8
	WORD		x, y;				// 0,0
	WORD		width, height;		// 23x23 / size for a 24x24
	WORD		horz_res, vert_res;	// 640x480 / screen x and y direction
	BYTE		ega_palette[48];	// ignore
	BYTE		reserved;			// ignore
	BYTE		num_color_planes;	// 8 / number of planes in image
	WORD		bytes_per_line;		// 24 (for a 24x24) bytes per one horizontal line
	WORD		palette_type;		// ignore
	BYTE		padding[58];		// extra space
	};

struct PCX_image
	{
	PCX_imageHeader	header;
	BYTE			pal[768];
	BYTE			buffer[];
	};

struct PCX_resourceHeader
	{
	DWORD	sizeofDecompressed;
	WORD	width,		// number of pixels across eg 24, unlike PCX_image
			height;		// header where width is eg 0-23
	};

struct PCX_resource
	{
	PCX_resourceHeader	header;
	BYTE				buffer[];
	};

//	-------------------------------------------------------------------
#define		PCX_HEADER_SIZE				128L
#define		PCX_RESOURCE_HEADER_SIZE	8L

#define		PCX_PALETTE_SIZE			768L

#define		pcx_CompressSizeOk			0
#define		pcx_CompressLargerthanImage	-99
#define		pcx_OutOfMemory				-98


// - Converting between data storages

// convert a PCX_image into a grImage
void	PCX_ToImage(grImage* dest, PCX_image* srce);
// convert a grImage into a PCX_image
long	PCX_FromImage(PCX_image* dest, grImage* srce);

// convert a PCX_resource into a grImage
void	PCX_ResourceToImage(grImage* dest, PCX_resource* srce);
// convert a grImage into a PCX_resource
long	PCX_ResourceFromImage(PCX_resource* dest, grImage* srce);

// convert a PCX_image into a PCX_resource
void	PCX_ToResource(PCX_resource* dest, PCX_image* srce,ulong buffer_size);


// - Compression/Decompression routines

// compress grImage data into PCX_image or PCX_resource buffer
long	PCX_Compress(char* dest, grImage* srce);
// decompress PCX_image or PCX_resource buffer into grImage data
void	PCX_Decompress(grImage* dest, char* srce);

// - Sizes from a PCX file

// Get size an image would be after extracting from a .PCX file
ulong	PCX_ImageSize(DirString* filename);
// Get width & height of a PCX file
int		PCX_GetWidthHeight(DirString* filename, uint* width, uint* height);

// - Loading and Saving a PCX file

// Load an image from a .PCX file
int		PCX_LoadImage(DirString* filename, grImage* dest, bool enable_palette = false);
// Save an image to a .PCX file
int		PCX_SaveImage(DirString* filename, grImage* srce);
// Load a palette from a .PCX file
grPalette*	PCX_LoadPalette(DirString* filename, bool enable_palette = false);
// Save a palette to a .PCX file
//int		PCX_SavePaletteToPCX (DirString* filename, palette* srce);

// Create a sprite from any number of PCX files with their image sizes equal
//sprite* PCX_MakeSpriteFromPCXs( DirString** filelist, int number_frames );

//	---------------------------------------------------------------------------

#endif
