//
//	REALMODE.CPP - Issues a real-mode interrupt from protected mode
//	using DPMI call 300h.
//
//	Note that any buffers to be passed to DOS _must_ be allocated in DOS memory.
//  This can be done with DPMI call 100h. (refer WATCOM\SRC\GOODIES\SIMULATE.C)
//

#include <i86.h>
#include <dos.h>
#include <stdio.h>
#include <conio.h>
#include <string.h>

#include "realmode.hpp"

rminfo RMI;

void dumpRMI(rminfo* rmi)
	{
	printf("\nRMI address       == %x:%x",FP_SEG(rmi),FP_OFF(rmi));
	printf("\nEDI == %.8x   ESI == %.8x   EBP == %.8x",rmi->EDI,rmi->ESI,rmi->EBP);
	printf("\nEAX == %.8x   EBX == %.8x   ECX == %.8x   EDX == %.8x",rmi->EAX,rmi->EBX,rmi->ECX,rmi->EDX);
	printf("\nflags == %.4x",rmi->flags);
	printf("\nES  == %.4x   DS  == %.4x   FS  == %.4x   GS  == %.4x",rmi->ES,rmi->DS,rmi->FS,rmi->DS);
	printf("\nIP  == %.4x   CS  == %.4x   SP  == %.4x   SS  == %.4x",rmi->IP,rmi->CS,rmi->SP,rmi->SS);
	printf("\n");
	}

void dumpREGS(REGS* dregs)
	{
	printf("\nREGS address      == %x:%x",FP_SEG(dregs),FP_OFF(dregs));
	printf("\nEDI == %.8x   ESI == %.8x",dregs->x.edi,dregs->x.esi);
	printf("\nEAX == %.8x   EBX == %.8x   ECX == %.8x   EDX == %.8x",dregs->x.eax,dregs->x.ebx,dregs->x.ecx,dregs->x.edx);
	printf("\ncflag == %.4x",dregs->x.cflag);
	printf("\n");
	}

void dumpSREGS(SREGS* dsregs)
	{
	printf("\nSREGS address     == %x:%x",FP_SEG(dsregs),FP_OFF(dsregs));
	printf("\nES  == %.4x   CS  == %.4x   SS  == %.4x   DS  == %.4x",dsregs->es,dsregs->cs,dsregs->ss,dsregs->ds);
	printf("\nFS  == %.4x   GS  == %.4x",dsregs->fs,dsregs->gs);
	printf("\n");
	}

void far* RM_malloc(short* selector,short* segment,unsigned bytes)
	{
	// temporary regs
	REGS regs;
	SREGS sregs;

	memset(&regs,0,sizeof(regs));
	memset(&sregs,0,sizeof(sregs));

	// use DPMI call 0x0100 (ALLOCATE DOS MEMORY)
	regs.w.ax = 0x0100;
	regs.w.bx = (bytes/16) + 1;		// # paragraphs to allocate (+1 for safety)

	// allocate
	int386x( DPMI_int, &regs, &regs, &sregs );

	// if !successful return NULL
//	if (regs.x.cflag) return NULL;

	// and return results
	*selector = regs.w.dx;
	*segment  = regs.w.ax;

	char far* p = (char far*) MK_FP(regs.w.dx,0);
	return p;
	}

void RM_free(short selector)
	{
	REGS regs;
	SREGS sregs;
	regs.w.ax = 0x0101;		// DPMI 1.0 free DOS memory
	regs.w.dx = selector;
	int386x( DPMI_int, &regs, &regs, &sregs );
	}

void RM_int86(int intno, union REGS *inregs, union REGS *outregs)
	{
	// temp storage for interrupt
	REGS regs;
	SREGS sregs;

//	printf("\nAttempting to generate a real-mode interrupt");
//	printf("\nusing myint86() via DPMI call 0300h");

	// set up real-mode call structure

//	memset(&RMI,0,sizeof(RMI));
	RMI.EDI = inregs->x.edi;
	RMI.ESI = inregs->x.esi;
	RMI.EAX = inregs->x.eax;
	RMI.EBX = inregs->x.ebx;
	RMI.ECX = inregs->x.ecx;
	RMI.EDX = inregs->x.edx;
	RMI.flags = inregs->x.cflag;

//	dumpRMI(&RMI);

	// Use DMPI call 300h to issue the DOS interrupt
//	memset(&regs,0,sizeof(regs));
//	memset(&sregs,0,sizeof(sregs));

	regs.w.ax = 0x0300;
	regs.h.bl = intno;
	regs.h.bh = 0;
	regs.w.cx = 0;
	sregs.es = FP_SEG(&RMI);
	regs.x.edi = FP_OFF(&RMI);

//	dumpREGS(&regs);
//	dumpSREGS(&sregs);
//	getch();

	// and call the interrupt

	int386x( DPMI_int, &regs, &regs, &sregs );

//	printf("\nDPMI function 0300h returned the following regs");

	// return our result...
//	memset(outregs,0,sizeof(outregs));
	outregs->x.edi = RMI.EDI;
	outregs->x.esi = RMI.ESI;
	outregs->x.eax = RMI.EAX;
	outregs->x.ebx = RMI.EBX;
	outregs->x.ecx = RMI.ECX;
	outregs->x.edx = RMI.EDX;
	outregs->x.cflag = RMI.flags;

//	dumpRMI(&RMI);
//	dumpREGS(&regs);
//	dumpREGS(outregs);
//	dumpSREGS(&sregs);
//	getch();
	}

void RM_int86x(int intno, union REGS *inregs, union REGS *outregs,struct SREGS* insregs)
	{
	REGS regs;
	SREGS sregs;

//	printf("\nAttempting to generate a real-mode interrupt");
//	printf("\nusing myint86x() via DPMI call 0300h");

	// set up real-mode call structure based upon inregs
//	memset(&RMI,0,sizeof(RMI));
	RMI.EDI = inregs->x.edi;
	RMI.ESI = inregs->x.esi;
	RMI.EAX = inregs->x.eax;
	RMI.EBX = inregs->x.ebx;
	RMI.ECX = inregs->x.ecx;
	RMI.EDX = inregs->x.edx;
	RMI.flags = inregs->x.cflag;
	RMI.ES  = insregs->es;
	RMI.CS  = insregs->cs;
	RMI.SS  = insregs->ss;
	RMI.DS  = insregs->ds;

//	dumpRMI(&RMI);

	// Use DMPI call 300h to issue the DOS interrupt
//	memset(&regs,0,sizeof(regs));
//	memset(&sregs,0,sizeof(sregs));

	regs.w.ax = 0x0300;
	regs.h.bl = intno;
	regs.h.bh = 0;
	regs.w.cx = 0;
	sregs.es = FP_SEG(&RMI);
	regs.x.edi = FP_OFF(&RMI);

//	dumpREGS(&regs);
//	dumpSREGS(&sregs);

	int386x( DPMI_int, &regs, &regs, &sregs );

//	printf("\nDPMI function 0300h returned the following regs");

	// return our result...
//	memset(outregs,0,sizeof(outregs));
	outregs->x.edi = RMI.EDI;
	outregs->x.esi = RMI.ESI;
	outregs->x.eax = RMI.EAX;
	outregs->x.ebx = RMI.EBX;
	outregs->x.ecx = RMI.ECX;
	outregs->x.edx = RMI.EDX;
	outregs->x.cflag = RMI.flags;
	insregs->es = RMI.ES;
	insregs->cs = RMI.CS;
	insregs->ss = RMI.SS;
	insregs->ds = RMI.DS;

//	dumpREGS(outregs);
//	getch();
	}


