//	-------------------------------------------------------------------------
//
//	STRINGS.HPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file contains code for a simple string class
//
//	-------------------------------------------------------------------------

#include "strings.hpp"
#include "math.h"

//	-------------------------------------------------------------------------
//
//	FUNC:		Set(long)
//	DESC:       Sets the string to represent a numerical value
//
//	-------------------------------------------------------------------------
void String::Set(long c)
	{
	long cc = c;
	long tmp = 100000000L;
	len = 0;
	// check for negative number
	if (c<0)
		{
		str[len++] = 45; // "-"
		cc = fabs(cc);
		}

	bool leadingzero = true;
	// convert number to string, including zeros
	while ((tmp!=0)&&(len<maxlen))
		{
		str[len] = 48 + cc / tmp;
		if (str[len]!=48) leadingzero = false;

		if (!leadingzero) len++;
		cc = cc - (cc/tmp) * tmp;
		tmp = tmp / 10;
		}

	if (len==0) { len = 1; str[0] = 48; } // if blank string, set to "0"
	str[len] = 0x00;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		StringToInt
//	DESC:       converts a string to an integer value, ignoring whitespace
//
//	RETURNS:	int	- the value of the string
//
//	-------------------------------------------------------------------------
int String::StringToInt()
	{
	bool minus = false;
	int tmp = 0;
	int cnt = 0;
	while (cnt<Length())
		{
		if (str[cnt]==45) minus = true;
		if ((str[cnt]>=48)&&(str[cnt]<=57)) tmp = tmp*10 + (str[cnt]-48);
		cnt++;
		}
	if (minus) tmp = -tmp;
	return tmp;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		StringToInt
//	DESC:       converts a string to an integer value, ignoring whitespace
//				but only when the value is between min & max.
//
//	PARAMS:		min	- the minimum value
//				max - the maximum value
//
//	RETURNS:	int	- the value of the string
//
//	-------------------------------------------------------------------------
int String::StringToInt(int min,int max)
	{
	long tmp = StringToInt();
	if (tmp<min) { tmp=min; Set(tmp); }
	if (tmp>max) { tmp=max; Set(tmp); }
	return tmp;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Insert
//	DESC:       inserts a series of chars into the String.
//
//	PARAMS:		pos	- the position in the string to insert the chars
//				st	- the char array to insert
//
//	-------------------------------------------------------------------------
void String::Insert(uint pos,const char* st)
	{
	int stlen = strlen(st);
	for (int i=0;i<stlen;i++)
		Insert( pos+i, st[i] ); // note Insert(pos,char) checks insertion
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Insert
//	DESC:       inserts a single char into the String.
//
//	PARAMS:		pos	- the position in the String to insert the char
//				ch	- the character to insert
//
//	-------------------------------------------------------------------------
void String::Insert(uint pos,char ch)
	{
//	if (pos<0) return;
	if ((pos<maxlen)&&(Length()<maxlen))
		{
		for (uint i=len;i>pos;i--)
			str[i] = str[i-1];
		str[pos] = ch; len++;
		str[len] = 0x00;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Delete
//	DESC:       deletes chars from within a String
//
//	PARAMS:		pos			- the position to delete from
//				numchars	- the number of chars to delete
//
//	-------------------------------------------------------------------------
void String::Delete(uint pos,uint numchars)
	{
//	if ((pos>Length())||(pos<0)) return;
	if (pos>Length()) return;

	for (int ii=0;ii<numchars;ii++)
		for (int i=pos;i<len;i++)
			{
			str[i] = str[i+1];
			}

	len -= numchars;
	str[len] = 0x00;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Append
//	DESC:       appends a char array to the end of the String.
//
//	PARAMS:		st	- the char array to add
//
//	-------------------------------------------------------------------------
void String::Append(const char* st)
	{
	int n = 0;
	while (st[n])
		Append(st[n++]); // note: Append(char) checks for overflow
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Append
//	DESC:       appends a single char to the end of the String.
//
//	PARAMS:		ch	- the char to add
//
//	-------------------------------------------------------------------------
void String::Append(char ch)
	{
	if (Length()<maxlen)
		{
		str[len] = ch;
		len++;
		str[len] = 0x00;
		}
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		Copy
//	DESC:       copies a number of chars from the String to the String
//
//	PARAMS:		pos			- the position to copy from
//				newpos		- the position to copy to
//				numchars	- the number of chars to copy
//
//	-------------------------------------------------------------------------
void String::Copy(uint pos,uint newpos,uint numchars)
	{
	if ((newpos>=maxlen)||(pos>=Length())) return;

	int n = 0;
	while ((n<numchars)&&(len<maxlen))
		{
		str[newpos++] = str[pos++];
		n++; if (newpos>len) len++;
		}
	str[len] = 0x00;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ScanRight
//	DESC:       scans for a char from a position in the string to the end
//				of the string
//
//	PARAMS:		pos		- the position to search from
//				token	- the char we are searching for
//
//	RETURNS:	int		- the position that the token was found
//
//	-------------------------------------------------------------------------
int String::ScanRight(int pos,char token)
	{
	if ((Length()==0) || (pos>len)) return len;
	uint cnt = pos+1;
	while ((str[cnt]!=token)&&(cnt<len)) cnt++;
	return cnt;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		ScanLeft
//	DESC:       scans for a char from a position in the string to the start
//				of the string
//
//	PARAMS:		pos		- the position to search from
//				token	- the char we are searching for
//
//	RETURNS:	int		- the position that the token was found
//
//	-------------------------------------------------------------------------
int String::ScanLeft(int pos,char token)
	{
	if ((Length()==0) || (pos<0)) return 0;
	uint cnt = pos-1;
	while ((str[cnt]!=token)&&(cnt>0)) cnt--;
	return cnt;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		NumElems
//	DESC:       returns the number of elements in the string delimeted by
//				a char token.
//				eg. String* cal = {"JAN/FEB/MAR/APR/.../DEC"}
//					NumElems("/") == 12
//
//	PARAMS:		token	- the token which is used to differentiate elems
//
//	RETURNS:	int		- the number of elements found in the string
//
//	-------------------------------------------------------------------------
int String::NumElems(char token)
	{
	int tokcnt = 0;
	int strptr = 0;
	// simply scan through the string counting tokens
	do
		{
		if (str[strptr++]==token) tokcnt++;
		}
	// till we reach null
	while (str[strptr]);
	// and return the number of tokens found --> number of elements
	return tokcnt;
	}


//	-------------------------------------------------------------------------
//
//	FUNC:		Elems
//	DESC:       returns a String containing a specified element.
//				eg. String* cal = {"JAN/FEB/MAR/APR/.../DEC"}
//					String* jan = cal->Elem(1,"/");
//					jan == "JAN"
//
//	PARAMS:		n		- the element number to return
//				token	- the token which is used to differentiate elems
//
//	RETURNS:	String*	- a new String containing the element
//
//	-------------------------------------------------------------------------
String* String::Elem(int n,char token)
	{
	int numelems = NumElems(token);
	String* tmpstr = NULL;

	int tokcnt = 0;
	int pos = 0;
	int start = 0;
	int end = 0;

	// if we have an appropriate element!!!
	if ((n<=numelems)&&(n>=0))
		{
		// if we are looking for the first element, simple
		if (n==0)
			{
			end = ScanRight(pos,token) - 1;
			}
		else
			// same for the last element
			if (n==numelems)
				{
				end = Length();
				start = ScanLeft(len,token) + 1;
				}
			else
				// scan through the string & set appropriate parameters
				{
				while (tokcnt<n)
					{
					pos = ScanRight(pos,token);
					tokcnt++;
					}
				start = pos + 1;
				end = ScanRight(pos,token) - 1;
				}
		// create & return appropriate string
		tmpstr = SubStr(start,end);
		}
	else
		{
		// return a NULL string!
		tmpstr = new String("");
		}

	return tmpstr;
	}

//	-------------------------------------------------------------------------
//
//	FUNC:		SubStr
//	DESC:       returns a String containing a substring of the string
//
//	PARAMS:		start	- the starting char
//				end		- the end char
//
//	RETURNS:	String*	- a new String containing the substring
//
//	-------------------------------------------------------------------------
String* String::SubStr( int start, int end )
	{
	String* buf = NULL;
	if (start>Length()) // return a zero-length string
		{
		buf = new String("");
		}
	else
		{
		// create a new string of appropriate length
		buf = new String(end-start+1);
		int strptr = start;

		// copy the string.. Note: use Append(char) because it error checks
		while ((str[strptr])&&(strptr<=end))
			{
			buf->Append(str[strptr++]);
			}
		}

	return buf;
	}


