//	-------------------------------------------------------------------------
//
//	STRINGS.HPP
//
//	-------------------------------------------------------------------------
//
//	DESCRIPTION:	This file contains defines for a simple string class
//
//	-------------------------------------------------------------------------

#ifndef __strings_hpp
#define __strings_hpp

#include "stddefns.hpp"

#include "stdio.h"
#include "conio.h"
#include "string.h"
#include "malloc.h"

// the default length of a String is 256 chars

#define STRING_DEFAULT_LENGTH	256

//	the String class

class String
	{
	protected:
		char* str;
		uint len;
		uint maxlen;
	public:
		// create a blank string of STRING_DEFAULT_LENGTH chars
		String()
				{
				len = 0;
				maxlen = STRING_DEFAULT_LENGTH;
				str = (char*) malloc(maxlen+1);
				str[0] = 0x00;
				}
		// create a blank string of a defined size
		String(uint size)
				{
				len = 0;
				maxlen = size;
				str = (char*) malloc(maxlen+1);
				str[0] = 0x00;
				}
		// create a string based upon a char*
		String(const char* st)
				{
				len = strlen(st);
				maxlen = len;
				str = (char*) malloc(maxlen+1);
				strcpy( str, st );
				str[maxlen] = 0x00;
				}
		// create a string based upon a char* of a specified size
		String(const char* st, uint size )
				{
				len = strlen(st);
				maxlen = size;
				str = (char*) malloc(maxlen+1);
				strncpy( str, st, maxlen );
				str[maxlen] = 0x00;
				}

		uint Length() { return len = strlen(str); }		// returns current length of string
		uint MaxLength() { return maxlen; }	// returns max. length of string
		void Set(const char* st)			// set the string to 'st'
				{
				if (strlen(st)<maxlen)
					{
					len = strlen(st);
					strcpy( str, st );
					}
				else
					{
					len = maxlen;
					strncpy( str, st, maxlen);
					str[len] = 0x00;
					}
				}

		void Set(char ch)					// set the string to ch
				{ str[0] = ch; len = 1; str[len] = 0x00; }

		void Set(long c);					//	set the string to a numeric value
		void Set(String* st) { Set(st->Contents()); }	// set the string to another string's

		void Insert(uint pos,const char* st);          	// insert a string at pos
		void Insert(uint pos,char ch);					// insert a char at pos
		void Delete(uint pos,uint numchars);			// delete numchars at pos
		void Append(const char* st);					// append st to the string
		void Append(char ch);							// append ch to the string
		void Copy(uint pos,uint newpos,uint numchars);	// copy numchars from pos to newpos
		char* Contents() { return str; }				// returns the actual contents of the string object
		operator char*() { return str; }				// same as Contents()

		int StringToInt();								// returns the integer value of the string
		int StringToInt(int min,int max);				// returns the integer value with min/max adjustment

		void Upper()	{ strupr(str); }				// converts string to uppercase
		void Lower()	{ strlwr(str); }				// converts string to lowercase

		// search string functions
		int		NumElems(char token);					// returns the number of elements in the string
		int		ScanRight(int pos,char token);			// returns the first occurrance of char after pos
		int		ScanLeft(int pos,char token);			// returns the first occurrance of char before pos

		// the following return new String objects - These must be DELETED after use
		String* Elem(int n,char token);					// returns string between n'th occurrence of token & (n+1)th occ.
		String* SubStr( int start, int end );			// returns a substring of the string.

		~String() { free(str); }
	};

#endif